/*
 * uc1701_v1.c
 *
 *  Created on: 24. 12. 2019
 *      Author: misa
 */

#include "uc1701_v1.h"

void uc1701_delay_us(uint16_t Delay);
void uc1701_delay_ms(uint16_t Delay);

// Inicializace periferií (GPIO)
// Inicializace Pinů SCK,MOSI,CS,CD(RS)

void init_bus(void){
	LL_GPIO_InitTypeDef gp;

	PORT_CLOCK_ENABLE; // makro z .h

	gp.Pin = UC1701_CS_PIN;
	gp.Mode = LL_GPIO_MODE_OUTPUT;
	gp.Pull = LL_GPIO_PULL_NO;
	gp.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
	gp.Speed = LL_GPIO_SPEED_FREQ_MEDIUM;
	gp.Alternate = LL_GPIO_AF_5;
	LL_GPIO_Init(UC1701_CS_GPIO,&gp);

	UC1701_CS_H; // deaktivujeme čip

	gp.Pin = UC1701_RS_PIN;
	LL_GPIO_Init(UC1701_RS_GPIO,&gp);
	gp.Pin = UC1701_MOSI_PIN;
	LL_GPIO_Init(UC1701_MOSI_GPIO,&gp);
	gp.Pin = UC1701_SCK_PIN;
	LL_GPIO_Init(UC1701_SCK_GPIO,&gp);

}

// inicializace displeje
void uc1701_init(void){
init_bus(); // inicializace GPIO a SPI
uc1701_command(LCD_RESET); // příkaz k restartu
uc1701_delay_ms(6); // minimální doba pro dokončení resetu

uc1701_command(LCD_DISPLAY_DISABLE); // spustit displej
uc1701_command(LCD_START_LINE | 0); // scroll na nulu
uc1701_command(LCD_SEGDIR | 0); // zrcadlení podle dlouhé strany (128) musí být vypnuté (RAM má 132px)
uc1701_vertical_flip_off();
uc1701_invert_off();
uc1701_command(LCD_BIAS | 0); // kouzelná konstanta - NECHAT pro JLX12864g (a mnohé další)
uc1701_command(LCD_POWER_CONTROL | 0x07); // zapnout šťávu
uc1701_contrast(0x06); // kontrast
uc1701_command(LCD_VOLUME); // něco jako kontrast (dvoubajtový příkaz)
#ifdef UC1701
uc1701_command(0x03); // pro JLX12864G
#else
uc1701_command(0x20); // argument předchozího (výchozí / střední hodnota)
#endif

uc1701_command(LCD_DISPLAY_ENABLE); // spustit displej
}

// Přepíše obsah obrazové RAM v UC1701.
// argument je pole o rozměru FRAME_HEIGHT*FRAME_WIDTH Bytů (1kB pro 128x64px displej)
void uc1701_display(uint8_t* array){
	uint8_t i,j,mask;
	// pro všechny řádky (pages)
	for(i=0;i<FRAME_HEIGHT;i++){
	uc1701_command(LCD_COL_ADDRESS_MSB | 0); // adresa sloupců na 0
	uc1701_command(LCD_COL_ADDRESS_LSB | 0);
	uc1701_command(LCD_PAGE_ADDRESS | i); // adresa příslušného řádku (page)
	UC1701_SCK_H;
	UC1701_RS_H; // budeme zapisovat data
	UC1701_CS_L; // aktivujeme SPI slave
	// pro všechny sloupce
	for(j=0;j<FRAME_WIDTH;j++){
		mask=1<<7;
		while(mask){
			if((*array) & mask){UC1701_MOSI_H;}else{UC1701_MOSI_L;}
			UC1701_SCK_L;
			mask = mask>>1;
			UC1701_SCK_H;
		}
		array++;
	}
	// deaktivujeme slave
	UC1701_CS_H;
	// a jdeme na další řádek
	}
}


// Odešle 1B příkazu (CD v log.0)
void uc1701_command(uint8_t command){
uint8_t mask=1<<7;
	UC1701_SCK_H;
	UC1701_RS_L; // command
	UC1701_CS_L;
	while(mask){
		UC1701_SCK_L;
		if(command & mask){UC1701_MOSI_H;}else{UC1701_MOSI_L;}
		mask = mask>>1;
		UC1701_SCK_H;
	}
	UC1701_CS_H;
}

void uc1701_create_test_pattern(uint8_t* image_buffer){
	uint8_t i,j,k=0;
	for(i=0;i<FRAME_HEIGHT;i++){
		for(j=0;j<FRAME_WIDTH;j++){
			image_buffer[j+i*FRAME_WIDTH] = k;
			k++;
		}
	}
}


void uc1701_delay_ms(uint16_t Delay){
	while(Delay){
	uc1701_delay_us(1000);
	Delay--;
	}
}

/*
 * Coarse blocking "delay microsecond" function.
 * Need (free) running systick with period longer then any ISR routine
 * Jitter dominated by IRQ routines (for example systick routine)
 * Measured values for STM32G030 running 64MHz with 10us nominal delay
 * -Og 				11.9us (1us jitter)
 * -O1 				13.0us(2us jitter)
 * -O2 				10.2us (1us jitter)
 * -O3 				10.3us (0.2us jitter)
 * -Os 				11.7us (1us jitter)
 * -Ofast 	10.2us (0.2us jitter)
 */
void uc1701_delay_us(uint16_t Delay){
	int32_t delay_ticks = Delay*(SystemCoreClock/1000000UL);
	int32_t last = SysTick->VAL;
	int32_t elapsed = 0;
	while (elapsed < delay_ticks){
		int32_t now = SysTick->VAL;
		elapsed += (last - now);
		last = now;
		if(elapsed < 0) {
			elapsed += SysTick->LOAD;
		}
	}
}
